#include "hipmain.h"
#include "tycho2.h"
#include "photodis.h"
#include "loadsp.h"
#include <stdio.h>

const long  NHip = 117955; //   
const float RelErr = 0.5;  //    
const float Rmax   = 500.0; //    
typedef struct
{
  long  Hip;  //    Hipparcos
  float Plx;  //    mas
  float sPlx; //    mas
} THip;

THip HipCat[NHip];

void LoadHipparcos()
{ //    Hipparcos   HipCat
 long i;
 THipparcos s;

 OpenHipparcosMain();
 i=0;
 while (ReadHipparcosMain(&s))
 {
  if (s.Info.NoPlx) continue;
   HipCat[i].Hip=s.HIP;
   HipCat[i].Plx=s.Plx;
   HipCat[i].sPlx=s.sigma.Plx;
   i++;
 }
 CloseHipparcosMain();
} // LoadHipparcos


  bool InterChange(long m, long n)
  {
   //   m-  n-  HipCat
  THip t;
   t=HipCat[m]; HipCat[m]=HipCat[n]; HipCat[n]=t;
   return false;
  }


void SortHipparcos()
{ //    HipCat

 bool Sorted; //  -  
 long l, r, i;

 l=0; r=NHip-1;

 do {
  Sorted=true; //  

  for (i=l; i<r; i++)     //   
   if (HipCat[i].Hip>HipCat[i+1].Hip) Sorted=InterChange(i,i+1);
  r--;

  for (i=r-1; i>=l; i--)  //   
   if (HipCat[i].Hip>HipCat[i+1].Hip) Sorted=InterChange(i,i+1);
  l++;
 } while (!Sorted);

} // SortHipparcos

long FindHip(long Hip0)
{
 //    ,   Hip=Hip0
 long l, r, n;

 l=0; r=NHip-1;
 do {
  n=(l+r)/2;
  if (HipCat[n].Hip==Hip0) return n;
  if (HipCat[n].Hip<Hip0)  l=n+1; else r=n;
 }
 while (l<r);
 return -1;
}

void main()
{
 const long step = Tycho2NumOfStars / 80;
 long k;
 TTycho2 s;
 long h; //     Hipparcos
 float Mj,BVj; //    B-V   
 char  TClass,SClass,LClass; //  
 float r_px,r_ph; //      
 FILE *f; //  

 printf("Load Hipparcos     ... "); LoadHipparcos(); printf("Ok\n");
 printf("Sort Hipparcos     ... "); SortHipparcos(); printf("Ok\n");
 OpenTycho2();
 printf("Load Spectral Data ... "); LoadSpData(); printf("Ok\n");
 f=fopen("r.txt","wt");
 while (ReadTycho2(&s))
 {
  if (k++ % step == 0) printf("*");
  //    Hipparcos
  h=FindHip(s.HIP);
  if (h<0) continue; //    ,   
  if (HipCat[h].Plx<=0.0) continue; // "" 
  //   
  if (HipCat[h].sPlx/HipCat[h].Plx>RelErr) continue;
  r_px=1000.0/HipCat[h].Plx;
  if (r_px>Rmax) continue;
  //    
  if (s.magn.NoVT  || s.magn.NoBT) continue;
  //   
  if (!FindSpData(s.TYC1,s.TYC2,s.TYC3,TClass,SClass,LClass)) continue;
  //   
  Mj =s.VT-0.090*(s.BT-s.VT);  BVj=0.850*(s.BT-s.VT);
  if (!PhotoDistance(TClass,SClass,LClass,Mj,BVj,r_ph)) continue;
  if (r_ph>Rmax) continue;
  fprintf(f,"%6d%8.1f%8.1f\n",HipCat[h].Hip,r_px,r_ph);
 }
 fclose(f);
}
