MODULE HipMain

IMPLICIT NONE



!     Hipparcos 
CHARACTER(*), PARAMETER :: HipparcosName = 'D:\CATALOGS\HIPARCOS\hip_main.dat'
!    Hipparcos
INTEGER, PARAMETER ::  HipNumOfStars = 118218

INTEGER, PARAMETER :: u = 10 !  


TYPE  THipparcos
   SEQUENCE
      INTEGER(4) :: HIP !    Hipparocs 

      !   
      REAL(8) :: RAdeg,DEdeg !     
      REAL(8) :: Plx         !    mas 
      REAL(8) :: pmRa,pmDE   !   ma*cos(d)  md  mas/ 

      CHARACTER(1) :: AstroRef    !     

      !   
      REAL(4)  :: VMag    !  .    
      REAL(4)  :: B_V     !   B-V    

     !  . 
      REAL(8) :: sigma_RAdeg,sigma_DEdeg
      REAL(8) :: sigma_Plx
      REAL(8) :: sigma_pmRa,sigma_pmDE
     

      CHARACTER(10) Sp !    

      LOGICAL NoRaDe  !      
      LOGICAL NoPlx   !     
      LOGICAL Nopm    !      
      LOGICAL NoVMag  !      
      LOGICAL NoB_V   !      

END TYPE THipparcos

INTEGER(4), ALLOCATABLE :: List(:)
INTEGER(4) :: NList = 0

CONTAINS
  
 !      " " 
 SUBROUTINE OpenHipparcosMain
  OPEN(u, file = HipparcosName)
 END SUBROUTINE OpenHipparcosMain

 !    
 SUBROUTINE  CloseHipparcosMain
   CLOSE(u)
 END SUBROUTINE  CloseHipparcosMain


 LOGICAL FUNCTION ReadHipparcosMain(s)
 TYPE(THipparcos), INTENT(out) :: s
 CHARACTER(450) hs !    

 IF (EOF(u)) THEN 
   ReadHipparcosMain=.false.
   RETURN
  ELSE
   ReadHipparcosMain=.true.
 END IF


 READ(u,'(A450)') hs !    



  !    
    
  s.NoRaDe = .False.  
  s.NoPlx  =.False.    
  s.Nopm   =.False.    
  s.NoVMag =.False.   
  s.NoB_V  =.False.    

  !   12 ,   3- -   HIP 
  read(hs(3:14),*) s.hip

  !  :  12   52   65  
  
  IF (LEN(TRIM(hs(52:63))) == 0) THEN
    s.NoRaDe=.true. 
	s.RADeg=0.0
   ELSE
    READ(hs(52:63),*) s.RAdeg
  END IF

  IF (LEN(TRIM(hs(65:76))) == 0) THEN
    s.NoRaDe=.true. 
	s.DEDeg=0.0
   ELSE
    read(hs(65:76),*) s.DEdeg
  END IF

  !   - 7   80-  }
  IF (LEN(TRIM(hs(80:86))) == 0) THEN
    s.NoPlx=.true. 
	s.Plx=0.0
   ELSE
    read(hs(80:86),*) s.Plx
  END IF


  !   :  8   88   97  
  IF (LEN(TRIM(hs(88:95))) == 0) THEN
    s.NoPM=.true. 
	s.pmRA=0.0
   ELSE
    read(hs(88:95),*) s.pmRA
  END IF

  IF (LEN(TRIM(hs(88:95))) == 0) THEN
    s.NoPM=.true. 
	s.pmDE=0.0
   ELSE
    read(hs(97:104),*) s.pmDE
  END IF


  s.AstroRef=hs(78:78) !    
    

  !  .    B-V    

  IF (LEN(TRIM(hs(42:46))) == 0) THEN
    s.NoVMag=.true. 
	s.VMag=0.0
   ELSE
    read(hs(42:46),*) s.VMag
  END IF


  IF (LEN(TRIM(hs(246:251))) == 0) THEN
    s.NoB_V=.true. 
	s.B_V=0.0
   ELSE
    read(hs(246:251),*) s.B_V
  END IF


  IF (.NOT. s.NoRADE) THEN
    read(hs(106:111),*) s.sigma_RAdeg !     ,
	read(hs(113:118),*) s.sigma_DEdeg !     
  ENDIF

  IF (.NOT. s.NoPlx) THEN
   read(hs(120:125),*) s.sigma_Plx
  END IF

  IF (.NOT. s.Nopm) THEN
    read(hs(127:132),*) s.sigma_pmRA
	read(hs(134:139),*) s.sigma_pmDE
  END IF

  !       
  s.Sp=hs(436:445)

 END FUNCTION ReadHipparcosMain


  INTEGER(4) FUNCTION InitCriteria(name)
  CHARACTER (*) :: name
  INTEGER, PARAMETER :: t = 11   ! 
  INTEGER(4) ::  i !    for
  
  OPEN(t, file = name, mode='read') !   
  
  DO i=1,2 !     
    READ(t,*) 
  END DO

  READ(t,*) NList !    -   
  PRINT *,NList

  !     
   ALLOCATE(List(NList));
   

  DO i=4,12 !   4  12  
    READ(t,*) 
  END DO

  DO i=1,NList
   READ(t,'(2X,I12)') List(i)
  END DO
  
  CLOSE(t)
  InitCriteria=NList
  
  END FUNCTION InitCriteria

  !   
  SUBROUTINE ClearCriteria
   DEALLOCATE(List)  
   NList=0
  END SUBROUTINE ClearCriteria



  !  ,      
  LOGICAL FUNCTION InCelestia(n)
  INTEGER(4) :: n
  INTEGER(4) :: i
  InCelestia=.false.;    !     
  if (NList==0) return   !     -  
   
  DO i=1,NList  !     for 
     if (List(i)==n) then !      
      InCelestia=.true.
      EXIT !                 
	 END IF 
   END DO

  END FUNCTION  InCelestia






 END MODULE HipMain


 